@@ -8,12 +8,22 @@ use super::Editor;
 impl Editor {
     /// Execute an edit event
     pub fn exe(&mut self, ev: Event) -> Result<()> {
-        if !self.plugin_active {
-            let same_type = self.doc().event_mgmt.last_event.as_ref();
-            // As long as event isn't present and the same as this one, commit
-            if same_type.map(|e| e.same_type(&ev)) != Some(true) {
+        let multi_cursors = self.doc().secondary_cursors.len() > 0;
+        if !(self.plugin_active || self.pasting || self.macro_man.playing || multi_cursors) {
+            let last_ev = self.doc().event_mgmt.last_event.as_ref();
+            // If last event is present and the same as this one, commit
+            let event_type_differs = last_ev.map(|e1| e1.same_type(&ev)) != Some(true);
+            // If last event is present and on a different line from the previous, commit
+            let event_on_different_line = last_ev.map(|e| e.loc().y == ev.loc().y) != Some(true);
+            // Commit if necessary
+            if event_type_differs || event_on_different_line {
                 self.doc_mut().commit();
             }
+        } else if self.doc().event_mgmt.history.is_empty() {
+            // If there is no initial commit and a plug-in changes things without commiting
+            // It can cause the initial state of the document to be lost
+            // This condition makes sure there is a copy to go back to if this is the case
+            self.doc_mut().commit();
         }
         self.doc_mut().exe(ev)?;
         Ok(())
@@ -33,9 +43,10 @@ impl Editor {
         } else {
             let loc = self.doc().char_loc();
             self.exe(Event::Insert(loc, ch.to_string()))?;
-            let file = &mut self.files[self.ptr];
-            if !file.doc.info.read_only {
-                file.highlighter.edit(loc.y, &file.doc.lines[loc.y]);
+            if let Some(file) = self.files.get_mut(self.ptr.clone()) {
+                if !file.doc.info.read_only {
+                    file.highlighter.edit(loc.y, &file.doc.lines[loc.y]);
+                }
             }
         }
         Ok(())
@@ -51,12 +62,13 @@ impl Editor {
             // Enter pressed in the start, middle or end of the line
             let loc = self.doc().char_loc();
             self.exe(Event::SplitDown(loc))?;
-            let file = &mut self.files[self.ptr];
-            if !file.doc.info.read_only {
-                let line = &file.doc.lines[loc.y + 1];
-                file.highlighter.insert_line(loc.y + 1, line);
-                let line = &file.doc.lines[loc.y];
-                file.highlighter.edit(loc.y, line);
+            if let Some(file) = self.files.get_mut(self.ptr.clone()) {
+                if !file.doc.info.read_only {
+                    let line = &file.doc.lines[loc.y + 1];
+                    file.highlighter.insert_line(loc.y + 1, line);
+                    let line = &file.doc.lines[loc.y];
+                    file.highlighter.edit(loc.y, line);
+                }
             }
         }
         Ok(())
@@ -78,15 +90,15 @@ impl Editor {
             // Backspace was pressed on the start of the line, move line to the top
             self.new_row()?;
             let mut loc = self.doc().char_loc();
-            let file = &self.files[self.ptr];
+            let file = self.files.get_mut(self.ptr.clone()).unwrap();
             if !file.doc.info.read_only {
                 self.highlighter().remove_line(loc.y);
             }
             loc.y = loc.y.saturating_sub(1);
-            let file = &mut self.files[self.ptr];
+            let file = self.files.get_mut(self.ptr.clone()).unwrap();
             loc.x = file.doc.line(loc.y).unwrap().chars().count();
             self.exe(Event::SpliceUp(loc))?;
-            let file = &mut self.files[self.ptr];
+            let file = self.files.get_mut(self.ptr.clone()).unwrap();
             let line = &file.doc.lines[loc.y];
             if !file.doc.info.read_only {
                 file.highlighter.edit(loc.y, line);
@@ -101,7 +113,7 @@ impl Editor {
                         y: self.doc().loc().y,
                     };
                     self.exe(Event::Delete(loc, ch.to_string()))?;
-                    let file = &mut self.files[self.ptr];
+                    let file = self.files.get_mut(self.ptr.clone()).unwrap();
                     if !file.doc.info.read_only {
                         file.highlighter.edit(loc.y, &file.doc.lines[loc.y]);
                     }
@@ -121,9 +133,10 @@ impl Editor {
                     y: self.doc().loc().y,
                 };
                 self.exe(Event::Delete(loc, ch.to_string()))?;
-                let file = &mut self.files[self.ptr];
-                if !file.doc.info.read_only {
-                    file.highlighter.edit(loc.y, &file.doc.lines[loc.y]);
+                if let Some(file) = self.files.get_mut(self.ptr.clone()) {
+                    if !file.doc.info.read_only {
+                        file.highlighter.edit(loc.y, &file.doc.lines[loc.y]);
+                    }
                 }
             }
         }
