@@ -59,7 +59,6 @@ fn main() {
         panic!("{err:?}");
     }
 }
-
 /// Run the editor
 #[allow(clippy::too_many_lines)]
 fn run(cli: &CommandLineInterface) -> Result<()> {
@@ -132,7 +131,8 @@ fn run(cli: &CommandLineInterface) -> Result<()> {
             let mut highlighter = file_type.get_highlighter(&ged!(&editor).config, tab_width);
             highlighter.run(&ged!(mut &editor).get_doc(c).lines);
             let mut editor = ged!(mut &editor);
-            let file = editor.files.get_mut(c).unwrap();
+            let current_ptr = editor.ptr.clone();
+            let file = &mut editor.files.get_atom_mut(current_ptr).unwrap().0[c];
             file.highlighter = highlighter;
             file.file_type = Some(file_type);
         }
@@ -140,7 +140,8 @@ fn run(cli: &CommandLineInterface) -> Result<()> {
         ged!(mut &editor).next();
     }
     // Reset the pointer back to the first document
-    ged!(mut &editor).ptr = 0;
+    let current_ptr = ged!(mut &editor).ptr.clone();
+    ged!(mut &editor).files.move_to(current_ptr, 0);
 
     // Handle stdin if applicable
     if cli.flags.stdin {
@@ -148,7 +149,8 @@ fn run(cli: &CommandLineInterface) -> Result<()> {
         let mut holder = ged!(mut &editor);
         holder.blank()?;
         let this_doc = holder.doc_len().saturating_sub(1);
-        let doc = holder.get_doc(this_doc);
+        let current_ptr = holder.ptr.clone();
+        let doc = &mut holder.files.get_atom_mut(current_ptr).unwrap().0[this_doc].doc;
         doc.exe(Event::Insert(Loc { x: 0, y: 0 }, stdin))?;
         doc.load_to(doc.size.h);
         let lines = doc.lines.clone();
@@ -337,7 +339,7 @@ fn handle_lua_error(key_str: &str, error: RResult<(), mlua::Error>, feedback: &m
 
 /// Handle opening files
 fn handle_file_opening(editor: &AnyUserData, result: Result<()>, name: &str) {
-    // TEMPORARY WORK-AROUND: Delete after Rust 1.83
+    // Block any directories from being opened (we'll wait until file tree is implemented)
     if file_or_dir(name) == "directory" {
         fatal_error(&format!("'{name}' is a directory, not a file"));
     }
@@ -345,30 +347,25 @@ fn handle_file_opening(editor: &AnyUserData, result: Result<()>, name: &str) {
         Ok(()) => (),
         Err(OxError::AlreadyOpen { .. }) => {
             let len = ged!(&editor).files.len().saturating_sub(1);
-            ged!(mut &editor).ptr = len;
-        }
-        Err(OxError::Kaolinite(kerr)) => {
-            match kerr {
-                KError::Io(ioerr) => match ioerr.kind() {
-                    ErrorKind::NotFound => fatal_error(&format!("File '{name}' not found")),
-                    ErrorKind::PermissionDenied => {
-                        fatal_error(&format!("Permission to read file '{name}' denied"));
-                    }
-                    /*
-                    // NOTE: Uncomment when Rust 1.83 becomes stable (io_error_more will be stabilised)
-                    ErrorKind::IsADirectory =>
-                        fatal_error(&format!("'{name}' is a directory, not a file")),
-                    ErrorKind::ReadOnlyFilesystem =>
-                        fatal_error(&format!("You are on a read only file system")),
-                    ErrorKind::ResourceBusy =>
-                        fatal_error(&format!("The resource '{name}' is busy")),
-                    */
-                    ErrorKind::OutOfMemory => fatal_error("You are out of memory"),
-                    kind => fatal_error(&format!("I/O error occured: {kind:?}")),
-                },
-                _ => fatal_error(&format!("Backend error opening '{name}': {kerr:?}")),
-            }
+            let current_ptr = ged!(&editor).ptr.clone();
+            ged!(mut &editor).files.move_to(current_ptr, len);
         }
+        Err(OxError::Kaolinite(kerr)) => match kerr {
+            KError::Io(ioerr) => match ioerr.kind() {
+                ErrorKind::NotFound => fatal_error(&format!("File '{name}' not found")),
+                ErrorKind::PermissionDenied => {
+                    fatal_error(&format!("Permission to read file '{name}' denied"));
+                }
+                ErrorKind::IsADirectory => {
+                    fatal_error(&format!("'{name}' is a directory, not a file"));
+                }
+                ErrorKind::ReadOnlyFilesystem => fatal_error("You are on a read only file system"),
+                ErrorKind::ResourceBusy => fatal_error(&format!("The resource '{name}' is busy")),
+                ErrorKind::OutOfMemory => fatal_error("You are out of memory"),
+                kind => fatal_error(&format!("I/O error occured: {kind:?}")),
+            },
+            _ => fatal_error(&format!("Backend error opening '{name}': {kerr:?}")),
+        },
         result => fatal_error(&format!("Error opening file '{name}': {result:?}")),
     }
 }
